Module: DependentSelect::IncludesHelper
- Defined in:
- lib/dependent_select/includes_helper.rb
Instance Method Summary collapse
-
#dependent_select_includes(options = {}) ⇒ Object
returns html necessary to load the javascript needed for dependent_select.
Instance Method Details
#dependent_select_includes(options = {}) ⇒ Object
returns html necessary to load the javascript needed for dependent_select
3 4 5 6 7 8 |
# File 'lib/dependent_select/includes_helper.rb', line 3 def dependent_select_includes( = {}) return "" if @ds_already_included @ds_already_included=true javascript_include_tag("dependent_select/dependent_select") end |